<?

/***                        
                                 

                                                                                                                
                                                                                   

                                                                                                

                                                  
                                                  

                                                         
                                            

                                                     
                       
                                                                        
                           
                                              
                                              

                                          

                                        

                        
***/

class DBSync {

var $sourceDb;
var $targetDb;

var $sourceTable;
var $targetTable;
var $tabCols;
var $colType;

var $debugLevel;

function DBSync(&$sourceDb, &$targetDb) {
$this->sourceDb =& $sourceDb;
$this->targetDb =& $targetDb;
$this->debugLevel = 0;
$this->colType = array();
}

function setTable($sourceTable, $tabCols, $targetTable = "") {
$this->sourceTable = $sourceTable;
$this->targetTable = ($targetTable)?$targetTable:$sourceTable;

if (sizeof($tabCols) == 0) $tabCols = $this->getColumns();
$this->tabCols = $tabCols;
}

function setColumnType($col, $type) {
$this->colType[$col] = $type;
}

function setDebugLevel($val) {
$this->debugLevel = $val;
}

function sync($cond = '') {
$this->_debug(1, "Synchronizing $this->sourceTable for '$cond'");
$this->_deleteRows($cond);
$rs = $this->_readRows($cond);
while ($row = $this->_getNextRow($rs)) {
$this->_debug(2, "Inserting 1 Row in $this->targetTable");
$this->_insertRow($row);
}
}

function getColumns() {
$ret = array();
$cols = $this->_descTable($this->sourceDb, $this->sourceTable);

foreach($cols as $col) {
$ret[] = $col["Field"];
}
return $ret;
}

function syncTable() {
$_srcCols = $this->_descTable($this->sourceDb, $this->sourceTable);
$_tgtCols = $this->_descTable($this->targetDb, $this->targetTable);

$srcCols = array();
$tgtCols = array();

//               /       
 for ($i = 0; $i < sizeof($_srcCols); $i++) {
$name = $_srcCols[$i]["Field"];
if (substr($name,0,5) == "quick") {
$srcCols[$name] = $_srcCols[$i];
}
}

for ($i = 0; $i < sizeof($_tgtCols); $i++) {
$name = $_tgtCols[$i]["Field"];
if (substr($name,0,5) == "quick") {
$tgtCols[$name] = $_tgtCols[$i];
}
}

//             /       
 foreach ($srcCols as $key => $value) {
if (isset($tgtCols[$key])) {
unset ($srcCols[$key]);
unset ($tgtCols[$key]);
}
}

//                        
 foreach($tgtCols as $column) {
$this->_removeColumn($this->targetDb, $this->targetTable, $column["Field"]);
}

//                
 foreach($srcCols as $column) {
$this->_addColumn($this->targetDb, $this->targetTable, $column);
}
}

function _addColumn($db = '', $tbl = '', $col = '') {
if (!$db) return;
if (!$tbl) return;
if (!$col) return;

$name = $col["Field"];
$null = ($col["Null"] == "YES") ? "":"NOT NULL";
$default = "DEFAULT '".$col["Default"]."'";
$extra = $col["Extra"];
$type = $col["Type"];

if (!$name || !$type) return;
$this->_debug(1, "Adding column '$name' to table '$tbl'");
$sql = "ALTER TABLE $tbl ADD column `$name` $type $null $default $extra";
$this->_debug(3, $sql);
$rs = $db->query($sql);
$this->_debug(4, $sql."; rsnum(".count($rs).")");

$sql = "ALTER TABLE $tbl ADD INDEX ($name)";
$this->_debug(3, $sql);
$rs = $db->query($sql);
$this->_debug(4, $sql."; rsnum(".count($rs).")");
}

function _removeColumn($db = '', $tbl = '', $col = '') {
if (!$db) return;
if (!$tbl) return;
if (!$col) return;

$this->_debug(1, "Dropping column '$col' from table '$tbl'");
$sql = "ALTER TABLE $tbl DROP $col";
$this->_debug(3, $sql);
$rs = $db->query($sql);
$this->_debug(4, $sql."; rsnum(".count($rs).")");
}

function _descTable($db = '', $tbl) {
if (!$db) return;
if (!$tbl) return;

$this->_debug(1, "Fetching table information for '$tbl'");
$sql = "DESCRIBE ".$tbl;
$this->_debug(3, $sql);
$rs = $db->query($sql);
$this->_debug(4, $sql."; rsnum(".count($rs).")");
return $rs;
}

function _debug($level, $msg) {
if ($level > $this->debugLevel) return;
echo date("Y-m-d H:i:s").":($level) ".$msg." \r\n";
}

function _deleteRows($cond = '') {
$sql = "DELETE FROM ".$this->targetTable;
if ($cond) $sql .= " WHERE $cond";
$this->_debug(3, $sql);
$this->targetDb->query($sql);
}

function _readRows($cond = '', $targetDb = false) {
$tab = $targetDb?$this->targetTable:$this->sourceTable;
$sql = "SELECT ".implode(",", $this->tabCols)." FROM $tab";
if ($cond) $sql .= " WHERE $cond";
if ($targetDb) $rs = $this->targetDb->query($sql);
else $rs = $this->sourceDb->query($sql);
$this->_debug(4, $sql."; rsnum(".count($rs).")");
return array("rs"=>$rs, "pos"=>-1);
}

function _getNextRow(&$rs) {
if (!is_array($rs) || !is_array($rs["rs"])) return false;
$rs["pos"]++;
if ($rs["pos"] >= count($rs["rs"])) return false;
return $rs["rs"][$rs["pos"]];
}

function _insertRow($row) {
$sql = "INSERT INTO ".$this->targetTable;
$cols = "";
$vals = "";
$i = 0;
foreach ($this->tabCols as $col) {
if (!isset($row[$col]) || $row[$col] == NULL) continue;
$cols .= $cols?",$col":"$col";
$vals .= $vals?",":"";
if ($this->colType[$col] == "blob") {
$this->targetDb->bindBlob(":bindvar$i", $row[$col], $col);
$vals .= ":bindvar$i";
$i++;
} else if ($this->colType[$col] == "clob") {
$this->targetDb->bindClob(":bindvar$i", $row[$col], $col);
$vals .= ":bindvar$i";
$i++;
} else {
$vals .= "'".$this->targetDb->escape($row[$col])."'";
}
}
if ($cols) {
$sql .= "($cols) VALUES ($vals)";
$this->_debug(3, $sql);
$this->targetDb->query($sql);
}
}
}

class DiffDBSync extends DBSync {

var $keyCol;
var $joinTable;

function DiffDBSync(&$sourceDb, &$targetDb, $keyCol) {
parent::DBSync($sourceDb, $targetDb);
$this->keyCol = $keyCol;
$this->joinTable = false;
}

function setJoinTable($joinKeyCol, &$joinTable) {
unset($this->joinTable);
$this->joinKeyCol = $joinKeyCol;
$this->joinTable =& $joinTable;
}

function sync($cond = '') {
//                    
 $srs = $this->_readRows($cond);
$foundKeys = array();
while ($row = $this->_getNextRow($srs)) {

$sqlKey = "'".$this->targetDb->escape($row[$this->keyCol])."'";
//                   
 $foundKeys[] = $this->keyCol." != ".$sqlKey;
//                              
 $keycond = (($cond != "")?$cond." AND ":"").$this->keyCol." = $sqlKey";

$this->_debug(2, "Checking Row $this->keyCol = $sqlKey");

if ($this->_compareRows($row, $this->_getNextRow($this->_readRows($keycond, true)))) continue;

//                      
 if (!is_object($this->joinTable)) continue;
$this->joinTable->sync($this->joinKeyCol." = $sqlKey");

//                                   
 $this->_debug(1, "Sync row with $this->keyCol = $sqlKey");
$this->_deleteRows($keycond);
$this->_insertRow($row);
}

//                  
 if (count($foundKeys) == 0) $foundKeys[] = "1 = 1";
$delcond = (($cond != "")?$cond." AND ":"").implode(" AND ",$foundKeys);
$rs = $this->_readRows($delcond, true);
while ($row = $this->_getNextRow($rs)) {
$sqlKey = "'".$this->targetDb->escape($row[$this->keyCol])."'";
$keycond = (($cond != "")?$cond." AND ":"").$this->keyCol." = ".$sqlKey;

$this->_debug(1, "Delete row $this->keyCol = $sqlKey");

//                      
 if (is_object($this->joinTable)) $this->joinTable->sync($this->joinKeyCol." = ".$sqlKey);

//                                   
 $this->_deleteRows($keycond);
}
}

/**
  *                                       
  **/
function _compareRows($sourceRow, $targetRow) {
if (is_array($sourceRow) != is_array($targetRow)) return false;
//                   
 if (!is_array($sourceRow)) return true;
if (count($sourceRow) != count($targetRow)) return false;
foreach ($sourceRow as $key=>$val) {
if ($targetRow[$key] != $val) return false;
}
return true;
}
}

class SyncDb {

var $blobs;
var $clobs;
var $error;
var $fields;

function SyncDb() {
$this->_resetLobs();
$this->error = false;
$this->fields = array();
}

function escape($sql) {	return $sql; }

function query($sql) { return array(); }

function bindBlob($name, &$val, $field = "") {
unset($this->blobs[$name]);
$this->blobs[$name] =& $val;
if ($field) $this->fields[$name] = $field;
}

function bindClob($name, &$val, $field = "") {
unset($this->clobs[$name]);
$this->clobs[$name] =& $val;
if ($field) $this->fields[$name] = $field;
}

function startTransaction() {	}

function commit($logoff = true) {	}

function logoff() { }

function _resetLobs() {
$this->blobs = array();
$this->clobs = array();
}

function _error($error) {
$this->error = $error;
echo "DB Error: $error \n";
die();
}

function _resetError() {
$this->error = false;
}

function isError() {
return $this->error;
}
}

class MysqlSyncDb extends SyncDb {

var $handle;

function MysqlSyncDb($server, $login, $pw, $db) {
parent::SyncDb();
$this->handle = mysql_connect($server, $login, $pw);
if ($db) mysql_select_db($db, $this->handle);
}

function escape($sql) {
return addslashes($sql);
}

function query($sql) {
$this->_resetError();

//            
 foreach ($this->blobs as $name=>$val) {
$sql = str_replace($name, "\"".$this->escape($val)."\"", $sql);
}
foreach ($this->clobs as $name=>$val) {
$sql = str_replace($name, "\"".$this->escape($val)."\"", $sql);
}

$cmd = strtolower(substr($sql, 0, 6));

$rs = mysql_query($sql, $this->handle);
if (!$rs) $this->_error("SQL: $sql; Error: ".mysql_error($this->handle));

$this->_resetLobs();

$ret = array();
if ($cmd == "select" || substr($cmd,0,4) == "desc" || substr($cmd,0,4) == "show") {
if ($rs) while ($row = mysql_fetch_array($rs, MYSQL_ASSOC)) $ret[] = $row;
}

return $ret;
}

function logoff() {
mysql_close($this->handle);
}
}

class Oci8SyncDb extends SyncDb {

var $handle;

function Oci8SyncDb($login, $pw, $db = "") {
parent::SyncDb();
$this->handle = ocilogon($login, $pw, $db);
if ($this->handle == false) {
$error = OCIError();
$this->_error((is_array($error)) ? $error['message'] : null);
}			
}

function escape($sql) {
//                                      
 $sql = str_replace("'","\\'", $sql);
return $sql;
}

function _escape($sql) {
$sql = str_replace("\\'","''", $sql);
//                                      
 return $sql;
}

function query($sql) {
$this->_resetError();

$sql = $this->_escape($sql);
$cmd = strtolower(substr($sql, 0, 6));

$lobs = array();
$fields = array();
foreach ($this->blobs as $name=>$val) {
$lobs[$name] = OCINewDescriptor($this->handle, OCI_D_LOB);
if ($this->fields[$name]) {
$sql = str_replace($name, "EMPTY_BLOB()", $sql);
$fields[$name] = $this->fields[$name];
} else {
$lobs[$name]->WriteTemporary($this->blobs[$name]);
}
}
if (count($fields)) {
$sql .= " RETURNING ".implode(",", $fields)." INTO ".implode(",", array_keys($fields));
}

$stm = ociparse($this->handle, $sql);
if ($stm == false) {
$this->_error("DB Error; SQL: $sql");
return array();
}

foreach ($this->clobs as $name=>$val) {
ocibindbyname($stm, $name, &$this->clobs[$name], -1);
}
foreach ($lobs as $name=>$lob) {
ocibindbyname($stm, $name, &$lobs[$name], -1, OCI_B_BLOB);
}

ociexecute($stm, OCI_DEFAULT);

if (count($fields)) {
foreach($fields as $name=>$field) {
$lobs[$name]->save($this->blobs[$name]);
}
}

$ret = array();
if ($cmd == "select") {
while (OCIFetchInto($stm,$arr,OCI_ASSOC+OCI_RETURN_NULLS+OCI_RETURN_LOBS)) {
$row = array();
foreach ($arr as $key=>$val) $row[strtolower($key)] = $val;
$ret[] = $row;
}
}

foreach ($lobs as $name=>$lob) {
$lob->free();
}
ocifreestatement($stm);
$this->_resetLobs();

return $ret;
}

function commit($logoff = true) {
ocicommit($this->handle);
if ($logoff) $this->logoff();
}

function logoff() {
ocilogoff($this->handle);
}
}

class PearSyncDb extends SyncDb {

}

?>
